#! /usr/bin/env python
# -*- coding: utf-8 -*-

import sys, re, time, argparse, Protein.index_list, checkFile
import numpy as np
from os import listdir
from os.path import isfile, join
from readFile import *
from chooseMethod import *
from Protein.extract_aaindex import AAIndex


def main(args):
    """The main process of generating nucleic acid composition features.
    :param args: an object of the arguments.
    """
    finalist = None
    if args.seqType == 'RNA' and args.encoding in ['Triplet','PseSSC','PseDPC']:
        with open(args.input) as f:
            '''The SC format of the input file is as follows (generated by ViennaRNA-2.4.8):
               >Sequence name
               An RNA sequence should be consist of AGCU
               Secondary structure
            '''
            fastas = get_rnasc_data(f)
    elif args.seqType == 'Protein' and args.encoding == 'PDT':
        fastas = args.input
    elif args.seqType == 'Protein' and args.encoding in Protein.index_list.POSSUM:
        finalist = checkPSSM(args.input, args.pssm)
        fastas = readFasta(args.input)
    else:
        fastas = readFasta(args.input)
    
    #file check
    if args.seqType == 'DNA' and args.encoding == "IDKmer" and args.positiveSource != None and args.negativeSource != None:
        args.positiveSource = readFasta(args.positiveSource)
        if not checkFile.checkDNA(args.positiveSource):
            sys.exit('Error,the character of positiveSource must be ACGT!')
        args.negativeSource = readFasta(args.negativeSource)
        if not checkFile.checkDNA(args.negativeSource):
            sys.exit('Error,the character of negativeSource must be ACGT!')

    #read phyche values
    phyche_vals = None
    if args.seqType == 'DNA':
        if args.encoding in ['DAC', 'DCC', 'DACC', 'PseDNC', 'PseKNC', 'PC-PseDNC-General', 'SC-PseDNC-General']:    
            phyche_vals = readPhycheValue(args.phyche_list_file, args.extra_index_file, 2, Protein.index_list.DNA, args.all_prop)
        elif args.encoding in ['TAC', 'TCC', 'TACC', 'PC-PseTNC-General', 'SC-PseTNC-General']:
            phyche_vals = readPhycheValue(args.phyche_list_file, args.extra_index_file, 3, Protein.index_list.DNA, args.all_prop)
    elif args.seqType == 'RNA':
        if args.encoding in ['DAC', 'DCC', 'DACC','PC-PseDNC-General', 'SC-PseDNC-General']:
            phyche_vals = readPhycheValue(args.phyche_list_file, args.extra_index_file, 2, Protein.index_list.RNA, args.all_prop)
    elif args.seqType == 'Protein':
        if args.encoding in ['AC', 'CC', 'ACC']:
            # 0 denotes that transform the data format to dict {acid: [phyche_vals]}
            phyche_vals = readPhycheValue(args.phyche_list_file, args.extra_index_file, 0, Protein.index_list.PROTEIN, args.all_prop)
        elif args.encoding in ['PAAC', 'APAAC']:
            phyche_vals = readPhycheValue(args.phyche_list_file, args.extra_index_file, 1, Protein.index_list.PROTEIN, args.all_prop)
        
    
    file_out = file(args.output,'w')
    # feature_vector = chooseMethod(fastas, seqType, encoding, kValue, weight, mismatch, delta, lag, phyche_list_file, extra_index_file, all_prop, cp, gap, neg_source, raactype, subtype, positiveSource, negativeSource)
    feature_vector = chooseMethod(fastas, args.seqType, args.encoding, args.kValue, args.positiveSource, args.negativeSource, args.mismatch, args.delta, args.lag, phyche_vals, args.phyche_list_file, args.all_prop, args.lamada, args.weight, args.oli, args.max_distance, args.cp, args.gap, args.nlag, args.train, args.label, args.subtype, args.gap_lambda, args.type, args.raactype, args.path, args.frequency_file, finalist, args.smoothing_window, args.sliding_window, args.Xi, args.Alpha)
    
    if args.header == 'T':
        feature_vector = list(feature_vector)
        feature_name = args.encoding.lower().replace("-","_")
        header = []
        for index in range(len(feature_vector[0])):
            header.append(feature_name + "_" + str(index))
        feature_vector.insert(0,header)

    np.savetxt(file_out, feature_vector, delimiter=",", fmt='%s')


if __name__ == '__main__':

    parse = argparse.ArgumentParser(description="DIFFUSER: Generating various numerical representation schemes for protein, dna or rna sequences",
                                    formatter_class=argparse.RawTextHelpFormatter)

    parse.add_argument('--input', required=True,
                       help="The input files in FASTA format.")
    parse.add_argument('--output', required=True,
                       help="The output files for storing feature vectors.")
    parse.add_argument('--seqType', required=True, choices=['Protein', 'DNA', 'RNA'],
                       help="The sequence type.")
    parse.add_argument('--encoding', type=str, choices=['Kmer', 'RevKmer', 'IDKmer', 'Mismatch', 'Subsequence', 'DAC', 'DCC', 'DACC', 'TAC', 'TCC', 'TACC',
                                                        'MAC', 'GAC', 'NMBAC', 'PseDNC', 'PseKNC', 'PC-PseDNC-General', 'PC-PseTNC-General', 'SC-PseDNC-General',
                                                        'SC-PseTNC-General', 'Triplet', 'PseSSC', 'PseDPC', 'DR', 'DP', 'EAAC', 'CKSAAP', 'DDE', 'GAAC', 'EGAAC',
                                                        'CKSAAGP', 'GDPC', 'GTPC', 'BINARY', 'AC', 'CC', 'ACC', 'PDT', 'PAAC', 'APAAC', 'Moran', 'Geary', 'NMBroto',
                                                        'CTDC', 'CTDT', 'CTDD', 'CTriad', 'KSCTriad', 'SOCNumber', 'QSOrder', 'KNNprotein', 'KNNpeptide', 'AAINDEX',
                                                        'BLOSUM62', 'ZSCALE', 'PseKRAAC', 'Disorder', 'DisorderB', 'DisorderC', 'SSEC', 'SSEB', 'ASA', 'TA', 'NUM',
                                                        'EBGW', 'LOGO', 'LOGO-P-Value', 'LOGO-BLOSUM62', 'AAC-PSSM', 'D-FPSSM', 'smoothed-PSSM', 'AB-PSSM', 'PSSM-composition',
                                                        'RPM-PSSM', 'S-FPSSM', 'DPC-PSSM', 'k-separated-bigrams-PSSM', 'tri-gram-PSSM', 'EEDP', 'TPC', 'EDP', 'RPSSM',
                                                        'Pse-PSSM', 'DP-PSSM', 'PSSM-AC', 'PSSM-CC', 'AADP-PSSM', 'AATP', 'MEDP'],
                        help="The encoding type")
    parse.add_argument('--header', type=str, default='T', choices=['T','F'],
                       help="wheather add headers for raw descriptors.")
    parse.add_argument('--kValue', type=int,
                       help="For Kmer, IDKmer, mismatch, subsequence ... methods. The k value of kmer.")
    parse.add_argument('--positiveSource',
                       help="The input positive source file in FASTA format for IDKmer.")
    parse.add_argument('--negativeSource',
                       help="The input negative source file in FASTA format for IDKmer.")
    parse.add_argument('--mismatch', type=int, 
                       help="For mismatch. The max value inexact matching. (m<k)")
    parse.add_argument('--delta', type=float, 
                       help="For subsequence method. The value of penalized factor. (0<=delta<=1)")
    parse.add_argument('--phyche_list_file',
                       help="The indices file user choose.\n"
                            "Default indices:\n"
                            "DNA dinucleotide: Rise, Roll, Shift, Slide, Tilt, Twist.\n"
                            "DNA trinucleotide: Dnase I, Bendability (DNAse).\n"
                            "RNA: Rise, Roll, Shift, Slide, Tilt, Twist.\n"
                            "Protein: Hydrophobicity, Hydrophilicity, Mass.")
    parse.add_argument('--extra_index_file', help="The user-defined indices file.\n")
    parse.add_argument('--all_index', dest='all_prop', action='store_true', help="Choose all physicochemical indices")
    parse.add_argument('--no_all_index', dest='all_prop', action='store_false',
                       help="Do not choose all physicochemical indices, default.")
    parse.set_defaults(all_prop=False)
    parse.add_argument('--lag', type=int, 
                       help="The value of lag.")
    parse.add_argument('--lamada', type=int, 
                       help="The value of lamada.")
    parse.add_argument('--weight', type=float, 
                       help="The value of weight.")
    parse.add_argument('--oli', type=int, default=2, choices=[2,3],
                       help="Choose one kind of Oligonucleotide for DNA: 2 represents dinucleotide and 3 represents trinucleotide;\n"
                       "3 represents trinucleotide.")
    parse.add_argument('--max_distance', type=int,
                       help="For DR and Distance Pair methods. The max distance value of DR and Distance Pair.")
    parse.add_argument('--cp', choices=['cp_13', 'cp_14', 'cp_19', 'cp_20'],
                       help="For Distance Pair method. The reduced alphabet scheme. Choose one of the four:\n"
                       "cp_13, cp_14, cp_19, cp_20 ")
    parse.add_argument('--gap', type=int,
                       help="The value of gap.")
    parse.add_argument('--nlag', type=int, 
                       help="The value of nlag.")
    parse.add_argument('--train',
                       help="The training data in FASTA format for KNNprotein or KNNpeptide.")
    parse.add_argument('--label',
                       help="The label file for KNNprotein or KNNpeptide.")
    parse.add_argument('--subtype', choices=['g-gap', 'lambda-correlation'], default='g-gap', 
                       help="the subtype of the descriptor type for PseKRAAC, default is 'g-gap'.")
    parse.add_argument('--gap_lambda', choices=[0, 1, 2, 3, 4, 5, 6, 7, 8, 9], type=int, default=1,
                       help="the gap value or lambda value for the 'g-gap' model or 'lambda-correlation' model in PseKRAAC.")
    parse.add_argument('--type', choices=['type1', 'type2', 'type3A', 'type3B', 'type4', 'type5', 'type6A', 'type6B', 'type6C',
                                 'type7', 'type8', 'type9', 'type10', 'type11', 'type12', 'type13', 'type14', 'type15',
                                 'type16'], default='type1', 
                       help="the descriptor type of PseKRAAC.")
    parse.add_argument("--raactype", help="the reduced amino acids cluster type for PseKRAAC, detailed value can be refered to Protein/index_list.py", type=int)
    parse.add_argument("--path", help="data file path used for 'SSEB(C)', 'Disorder(BC)', 'ASA' and 'TA' encodings")
    parse.add_argument("--frequency_file", help="the parameter used for LOGO-based features and is generated by twosamplelogo")
    parse.add_argument("--pssm", help="the pssm file path used for 'PSSM-based features'")
    parse.add_argument("--smoothing_window", type=int, help="The value of smoothing window used for smoothed-PSSM")
    parse.add_argument("--sliding_window", type=int, help="The value of sliding window")
    parse.add_argument("--Xi", type=int, help="The value of Xi used for Pse-PSSM")
    parse.add_argument("--Alpha", type=int, help="The value of Alpha used for DP-PSSM")

    args = parse.parse_args()

    print("Calculating "+ args.encoding + " ...")
    start_time = time.time()
    main(args)
    print("Done.")
    print("Used time: %.2fs" % (time.time() - start_time))
