#!/usr/bin/env python
#_*_coding:utf-8_*_
# Authors: Chris & Ruopeng Xie

import sys, os, re, shutil
pPath = re.sub(r'util$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
import readFile
import argparse


def generateSpineX(fastas, outDir, vsl2):
	"""
	Generate predicted protein disorder information by bysing spineX software.

	Parameters
	----------
	file : file
		the file, which include the protein sequences in fasta format.
	
	spineX: the spineX software path, the spineX software could be downloaded at http://sparks-lab.org/pmwiki/download/index.php

	outDir: output folder
	
	Returns
	-------
	a string: 
		A directory name, which include the predicted protein disorder information.
	"""

	if os.path.exists(outDir) == False:
		os.mkdir(outDir)

	for i in fastas:
		name, sequence = i[0].replace(".","").replace("|",""), i[1]
		with open(outDir + '/' + name, 'w') as f:
			f.write(sequence + '\n')
		with open(outDir + '/list', 'w') as ff:
			ff.write(name + '\n')

		#shutil.move(outDir + '/' + name + '.txt', outDir + '/' + name)

		myCmd = 'sudo perl ' + spineX  + ' ' + outDir + '/list ' + outDir + ' ' + outDir
		
		if os.path.exists(outDir + '/' + name + '.spXout') == False:
			os.system(myCmd)
			os.remove(outDir + '/' + name)
			os.remove(outDir + '/list')
			os.remove(outDir + '/' + name +'.mat')
	return outDir

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.", description="generate protein secondary structure profile")
	parser.add_argument("--file", required=True, help="protein sequence file in fasta format")
	parser.add_argument("--spineX", help="the path of spineX program")
	parser.add_argument("--out", help="output file")
	args = parser.parse_args()

	spineX = args.spineX if args.spineX != None else 'spX.pl'
	fastas = readFile.readFasta(args.file)
	outputDir = generateSpineX(fastas, args.out, spineX)
	print('The predicted disorder are stored in: ' + outputDir)
