#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Fule Liu, Nackel, luo, Hao Wu
"""
import util
import index_list
import sys

def make_pseknc_vector(sequence_list, k, w, lamada, phyche_value, pse_type, alphabet=index_list.DNA):
    """Generate the pseknc vector.
    :parameter pse_type: 0 denotes PseKNC, 1 denotes PC, 2 denotes SC
    """
    if k <= 0:
        #for PseKNC
        print "parameters kValue must be an inter and larger than 0. The default value of kValue is 3."
        k = 3
    if lamada <= 0:
        print "parameters lamada is required. The default value of lamada is 2."
        lamada = 2
    if w <= 0:
        print "parameters weight is required. The default value of weight is 0.1 ."
        w = 0.1
    kmer = util.make_kmer_list(k, alphabet)
    vector = []

    for sequence in sequence_list:
        if len(sequence[1]) < k or lamada + k > len(sequence[1]):
            error_info = "Sorry, the sequence length must be larger than " + str(lamada + k)
            sys.stderr.write(error_info)
            sys.exit(0)

        # Get the nucleotide frequency in the DNA sequence.
        fre_list = [util.frequency(sequence[1], str(key)) for key in kmer]
        fre_sum = float(sum(fre_list))

        # Get the normalized occurrence frequency of nucleotide in the DNA sequence.
        fre_list = [e / fre_sum for e in fre_list]

        # Get the theta_list.
        if pse_type == 1:
            theta_list = util.get_parallel_factor(k, lamada=lamada, sequence=sequence[1], phyche_value=phyche_value, alphabet=alphabet)
        elif pse_type == 2:
            theta_list = util.get_series_factor(k, lamada=lamada, sequence=sequence[1], phyche_value=phyche_value, alphabet=alphabet)
        elif pse_type == 0:
            #Pse_KNC
            theta_list = util.get_parallel_factor(k=2, lamada=lamada, sequence=sequence[1], phyche_value=phyche_value, alphabet=alphabet)

        theta_sum = sum(theta_list)

        # Generate the vector according the Equation 9.
        denominator = 1 + w * theta_sum

        temp_vec = [round(f / denominator, 3) for f in fre_list]
        for theta in theta_list:
            temp_vec.append(round(w * theta / denominator, 3))

        vector.append(temp_vec)

    return vector

