#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Fule Liu, Nackel, luo, Hao Wu
"""

import index_list,sys,util
from numpy import array
import numpy as np
from itertools import combinations, product
import multiprocessing

def getSubsequenceProfileByParallel(fastas, k, delta):
    if k <= 0:
        print "parameters kValue is required. The default value of k is 3."
        k = 3
    if delta == None:
        print "parameters delta is required. The default value of delta is 1."
        delta = 1
    elif delta > 1 or delta < 0:
        sys.exit("Error! delta should be greater than or equal to 0 and less than or equal to 1.")
    alphabet = index_list.DNA
    alphabet = list(alphabet)
    cpu_num = multiprocessing.cpu_count()   
    batches = constructPartitions([fasta[1] for fasta in fastas], cpu_num)
    pool = multiprocessing.Pool(cpu_num)
    results = []
    
    for batch in batches:
	test = getSubsequenceProfile(batch, alphabet, k, delta)
        temp=pool.apply_async(getSubsequenceProfile, (batch, alphabet, k, delta))
        results.append(temp)
    pool.close()
    pool.join()
    i = 1
    for temp in results:
        temp_X = temp.get()
        if len(temp_X) != 0:
            if i == 1:
                X = temp_X
            else:
                X = np.vstack((X,temp_X))
            i += 1
    return X
    
def constructPartitions(seq_list, cpu_num):
    seqs_num = len(seq_list)
    batch_num = seqs_num//cpu_num
    batches = []
    for i in range(cpu_num-1):
        batch = seq_list[i*batch_num:(i+1)*batch_num]
        batches.append(batch)
    batch=seq_list[(cpu_num-1)*batch_num:]
    batches.append(batch)
    return batches
    
def getSubsequenceProfile(seq_list, alphabet, k, delta):
    kmerdict = util.getKmerDict(alphabet, k)
    X=[]
    for sequence in seq_list:
        vector = getSubsequenceProfileVector(sequence, kmerdict, k, delta)
        X.append(vector)
    X=array(X)    
    return X

def getSubsequenceProfileVector(sequence, kmerdict, k, delta):      
    vector = np.zeros((1,len(kmerdict)))
    sequence = array(list(sequence))
    n = len(sequence)
    #index_lst = list(combinations(range(n), k))
    for subseq_index in combinations(range(n), k):
        subseq_index = list(subseq_index)
        subsequence = sequence[subseq_index]
        position = kmerdict.get(''.join(subsequence))     
        subseq_length = subseq_index[-1] - subseq_index[0] + 1
        subseq_score = 1 if subseq_length == k else delta**subseq_length    
        vector[0,position] += subseq_score
    #return list(vector[0])
    return [round(f, 4) for f in list(vector[0])]

