#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Fule Liu, Nackel, luo, Hao Wu
"""
import numpy as np
from itertools import combinations_with_replacement, permutations, product
import index_list

def get_pseaacdis_matrix(fastas, max_distance, reduce_alphabet_scheme):
    if max_distance <= 0:
        print 'parameters max_distance is required. The default value of max_distance is 3.'
        max_distance = 3
    max_distance = int(max_distance)
    if max_distance < 0 or max_distance > 10:
        print 'The max distance can not be negative integer and should be smaller than 11.'
        return False
    if reduce_alphabet_scheme == None:
        print 'parameters cp is required. There are four choices: cp_13, cp_14, cp_19, cp_20. The default value of c is cp_14.'
        reduce_alphabet_scheme = index_list.cp_14
    elif reduce_alphabet_scheme == 'cp_13':
        reduce_alphabet_scheme = index_list.cp_13
    elif reduce_alphabet_scheme == 'cp_14':
        reduce_alphabet_scheme = index_list.cp_14
    elif reduce_alphabet_scheme == 'cp_19':
        reduce_alphabet_scheme = index_list.cp_19
    elif reduce_alphabet_scheme == 'cp_20':
        reduce_alphabet_scheme = index_list.cp_20
    features = []
    for seq in fastas:
        vector = get_pseaacdis_vector(seq[1], reduce_alphabet_scheme, max_distance)
        features.append(vector)
    return features

def get_pseaacdis_dict(raaas_lst, k):
    '''
    :param raaas_lst: A list of reduced amino acid alphabet scheme.
    :param k: The length of pseudo amino acid composition.
    return: a pseaac_dis pattern dictionary.
    '''
    pseaacdis_lst = []
    pseaacdis_dict = {}
    if k == 2:
        part_pseaa = list(combinations_with_replacement(raaas_lst, 2))
        for element in part_pseaa:
            elelst = set(permutations(element, 2))
            pseaacdis_lst += elelst
        pseaacdis_lst.sort()

        for i in range(len(pseaacdis_lst)):
            a, b = pseaacdis_lst[i]
            for j in product(list(a), list(b)):
                pseaacdis_dict[j] = i
        return pseaacdis_dict
    elif k == 1:
        pseaacdis_lst = raaas_lst
        for i in range(len(pseaacdis_lst)):
            for j in list(pseaacdis_lst[i]):
                pseaacdis_dict[j] = i
        return pseaacdis_dict
    else:
        return False


def get_pseaacdis_vector_d(sequence, raaas_lst, distance):
    if distance == 0:
        pseaacdis_dict = get_pseaacdis_dict(raaas_lst, 1)
        sequence = list(sequence)
        vector = np.zeros((1, len(set(pseaacdis_dict.itervalues()))))
        for i in sequence:
            position = pseaacdis_dict.get(i)
            vector[0, position] += 1
        return [round(f,3) for f in vector[0] / sum(vector[0])]
    elif distance > 0:
        pseaacdis_dict = get_pseaacdis_dict(raaas_lst, 2)
        sequence = list(sequence)
        vector = np.zeros((1, len(set(pseaacdis_dict.itervalues()))))
        for i in range(len(sequence) - distance):
            a, b = sequence[i], sequence[i + distance]
            position = pseaacdis_dict.get((a, b))
            vector[0, position] += 1
        return [round(f, 3) for f in vector[0] / sum(vector[0])]
    else:
        return False


def get_pseaacdis_vector(sequence, raaas_lst, max_distance):
    if max_distance >= 0:
        for i in range(max_distance + 1):
            vector_tmp = get_pseaacdis_vector_d(sequence, raaas_lst, i)
            if i == 0:
                vector = vector_tmp
            else:
                vector = np.concatenate((vector_tmp, vector))
        return vector
    else:
        return False
