#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Fule Liu, Nackel, luo, Hao Wu
"""
import sys, os, subprocess

def pdt_cmd(inputfile, lamada):
    """Concatenation of pdt command.
    :param inputfile: the input sequence file in FASTA format.
    :param lamada: the value of parameter lamada.
    """
    full_path = os.path.realpath(__file__)
    if sys.platform.startswith('win'):
        pdt_cmd = '%s\\pdt\\pdt.exe' % os.path.dirname(full_path)
    else:
        pdt_cmd = '%s/pdt/pdt' % os.path.dirname(full_path)
        #os.chmod(pdt_cmd, stat.S_IRWXU)
        os.chmod(pdt_cmd, 0777)

    input_file = inputfile
    aaindex_file = '%s/pdt/aaindex_norm.txt' % os.path.dirname(full_path)
    lamada_param = str(lamada)
    output_file = ''.join([os.path.splitext(input_file)[0], '_temp', os.path.splitext(input_file)[1]])

    cmd = ''.join([pdt_cmd, ' ', input_file, ' ', aaindex_file, ' ', lamada_param, ' ', output_file])
    #print cmd
    subprocess.call(cmd, shell=True)
    return os.path.abspath(output_file)


def pdt(inputfile, lamada):
    """Execute pdt command and generate feature vectors.
    :param inputfile: the input sequence file in FASTA format.
    :param lamada: the value of parameter lamada.
    """
    #inputfile = os.path.abspath(inputfile)
    #cwd = os.getcwd()
    #os.chdir('./PDT')
    if lamada <= 0:
        lamada = 1
        print "parameters -l is required. The default value of -l is 1."
    if lamada < 1 or lamada > 15:
        sys.exit('The value of -l should be larger than 0 and smaller than 16.')
    output_file = pdt_cmd(inputfile, lamada)
    #os.chdir(cwd)

    vector_list = []
    with open(output_file, 'r') as f:
        for line in f:
            temp_list = line.strip().split('\t')
            vector = [round(float(elem), 3) for elem in temp_list]
            vector_list.append(vector)
    #print vector_list
    return vector_list