#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Fule Liu, Nackel, luo, Hao Wu
"""
import util
import index_list
import sys

def pseknc(fastas, k, w, l, phyche_list_file, extra_index_file=None, all_prop=False):
    """This is a complete process in PseKNC.

    :param k: int, the value of k-tuple.
    :param phyche_list: list, the input physicochemical properties list.
    :param extra_index_file: a file path includes the user-defined phyche_index.
    :param all_prop: bool, choose all physicochemical properties or not.
    """
    phyche_list=""
    if all_prop == 'True':
        all_prop = True
    elif extra_index_file is None and phyche_list_file == '':
        print "parameters phyche_list is required. The default value of phyche_list is ['Hydrophobicity', 'Hydrophilicity', 'Mass']."
        phyche_list=index_list.INDS_3_PROTEIN
    if l == '':
        print "parameters lamada is required. The default value of lamada is 2."
        l = 2
    if w == '':
        print "parameters weight is required. The default value of weight is 0.1 ."
        w = 0.1
    if phyche_list_file != '':
        phyche_list = util.read_index(phyche_list_file)
    alphabet = index_list.PROTEIN
    phyche_list = util.get_phyche_list(k, phyche_list,
                                  extra_index_file=extra_index_file, alphabet=alphabet, all_prop=all_prop)
    phyche_vals = util.get_aaindex(phyche_list)
    if extra_index_file is not None:
            phyche_vals.extend(util.get_ext_ind_pro(extra_index_file))

    # # Transform the data format to dict {acid: [phyche_vals]}.
    # phyche_keys = phyche_vals[0].index_dict.keys()
    # phyche_vals = [e.index_dict.values() for e in phyche_vals]
    # new_phyche_vals = zip(*[e for e in phyche_vals])
    # phyche_vals = {key: list(val) for key, val in zip(phyche_keys, new_phyche_vals)}
    return make_pseknc_vector(fastas, phyche_vals, k, w, l, alphabet)


def make_pseknc_vector(sequence_list, phyche_value, k=2, w=0.05, lamada=1, alphabet=index_list.DNA):
    """Generate the pseknc vector."""
    kmer = util.make_kmer_list(k, alphabet)
    vector = []

    for sequence in sequence_list:
        if len(sequence[1]) < k or lamada + k > len(sequence[1]):
            error_info = "Sorry, the sequence length must be larger than " + str(lamada + k)
            sys.stderr.write(error_info)
            sys.exit(0)

        # Get the nucleotide frequency in the DNA sequence.
        fre_list = [util.frequency(sequence[1], str(key)) for key in kmer]
        fre_sum = float(sum(fre_list))

        # Get the normalized occurrence frequency of nucleotide in the DNA sequence.
        fre_list = [e / fre_sum for e in fre_list]

        # Get the theta_list.

        theta_list = util.get_parallel_factor(k=k, lamada=lamada, sequence=sequence[1],
                                             phyche_value=phyche_value, alphabet=alphabet)

        theta_sum = sum(theta_list)

        # Generate the vector according the Equation 9.
        denominator = 1 + w * theta_sum

        temp_vec = [round(f / denominator, 3) for f in fre_list]
        for theta in theta_list:
            temp_vec.append(round(w * theta / denominator, 3))

        vector.append(temp_vec)

    return vector

