#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Nackel
"""
import util, sys
import numpy as np


def get_psedpc_matrix(fastas, k, l, w, pattern_list = ['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G']):
    '''This is a complete process in PseSSC, aim to generate feature vector.
    :param filename: Name of input file.
    :param k: The maximum distance between structure statuses.
    :param l: The highest counted rank (or tier) of the structural correlation along a RNA chain.
    :param w: The weight of theta, from 0.1 to 1.
    :param pattern_list: Structure statuses, default:['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G'].'''
    if k <= 0:
        print "parameters k is required. The default value of k is 0."
        k = 0
    if l <= 0:
        print "parameters l is required. The default value of l is 2."
        l = 2
    if w <= 0:
        print "parameters w is required. The default value of w is 0.1."
        w = 0.1
    features = []
    for seqss in fastas:
        vector = get_psedpc_vector(seqss, k, l, w, pattern_list)
        features.append(vector)
    return features


def get_psedpc_vector(seqss, n, r, w, pattern_list = ['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G']):
    '''This is a complete process in PseSSC, aim to generate feature vector.
    :param seqss: a seqss object.
    :param n: The distance between structure statuses. 0<=n<=lenth-1
    :param r: The highest counted rank (or tier) of the structural correlation along a RNA chain. r<length
    :param w: The wight of theta, from 0.1 to 1.
    :param pattern_list: Structure statuses, default:['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G'].
    '''
    correspseq = util.get_corresp_sequence(seqss)
    pattern = zip(list(seqss.sequence), list(correspseq))
    if n >= seqss.length:
        error_info = 'Error occured in ' + seqss.id + ', n should be less than the length of the sequence.\n'
        sys.stderr.write(error_info)
    elif r >= seqss.length:
        error_info = 'Error occured in ' + seqss.id + ', r should be less than the length of the sequence.\n'
        sys.stderr.write(error_info)
    else:
        vector = []
        vector = np.array(vector, ndmin = 2)
        for i in range(n+1):
            if i != 0:
                k = 2
            else:
                k = 1
            psedpcdict = util.get_pseknc_dict(pattern_list, k)
            vec_tmp = np.zeros((1, len(psedpcdict)))

            for j in xrange(len(pattern) - i):
                stem=[]
                if i != 0:
                    for x, y in pattern[j], pattern[j+i]:

                        if x == '.' or y == '.':
                            if x == '.':
                                stem.append(y)
                            else:
                                stem.append(x)
                        else:
                            stem.append(x + '-' + y)
                    stem_tuple = tuple(stem)
                    #print stem_tuple
                else:
                    for x,y in [pattern[j]]:
                        if x == '.' or y == '.':
                            if x == '.':
                                stem.append(y)
                            else:
                                stem.append(x)
                        else:
                            stem.append(x + '-' + y)
                    stem_tuple = tuple(stem)
                    #print stem_tuple
                position = psedpcdict.get(stem_tuple)
                vec_tmp[0, position] += 1
            vector = np.hstack((vector, vec_tmp))

        psedpcvec_tmp = vector[0]
        for i in range(1, r+1):
            psedpcvec_tmp = np.hstack((psedpcvec_tmp,(w * util.calculate_theta(seqss, i))))
        psedpcvec = psedpcvec_tmp / (sum(psedpcvec_tmp[10+100*n:])+1)
       # psedpcvec = psesscvec_tmp / sum(psesscvec_tmp)
        #return psedpcvec
        return [round(f,4) for f in psedpcvec]
