#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Nackel
"""
import sys
import numpy as np
import util
from itertools import combinations_with_replacement, permutations



def get_psessc_matrix(fastas, k, l, w, pattern_list = ['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G']):
    '''This is a complete process in PseSSC, aim to generate feature matrix.
    :param filename: Name of input file.
    :param k: The number of k adjacent structure statuses.
    :param l: The highest counted rank (or tier) of the structural correlation along a RNA chain.
    :param w: The weight of theta, from 0.1 to 1.
    :param pattern_list: Structure statuses, default:['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G'].
    :return: Feature matrix through PseSSC.
    '''
    if k <= 0:
        print "parameters k is required. The default value of k is 2."
        k = 2
    if l <= 0:
        print "parameters l is required. The default value of l is 2."
        l = 2
    if w <= 0:
        print "parameters w is required. The default value of w is 0.1."
        w = 0.1
    features = []
    for seqss in fastas:
        vector = get_psessc_vector(seqss, k, l, w, pattern_list)
        features.append(vector)
    return features


def get_psessc_vector(seqss, n, r, w, pattern_list = ['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G']):
    '''This is a complete process in PseSSC, aim to generate feature vector.
    :param seqss: a seqss object.
    :param n: The number of n adjacent structure statuses.
    :param r: The highest counted rank (or tier) of the structural correlation along an RNA chain.
    :param w: The wight of theta, from 0.1 to 1.
    :param pattern_list: Structure statuses, default:['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G'].
    :return: Feature vector through PseSSC.
    '''
    #psekncdict = get_pseknc_dict(pattern_list, k)

    if n > seqss.length or n <= 0:
        error_info = 'Error occured in ' + seqss.id + ', n should be less than the length of the sequence and large than 0.\n'
        sys.stderr.write(error_info)
    elif r >= seqss.length:
        error_info = 'Error occured in ' + seqss.id + ', r should be less than the length of the sequence.\n'
        sys.stderr.write(error_info)
    else:
        psekncvec = util.get_pseknc_vector(seqss, n, pattern_list)

        psesscvec_tmp = np.array(psekncvec)
        for i in range(1, r+1):
            psesscvec_tmp = np.hstack((psesscvec_tmp, w * util.calculate_theta(seqss, i)))
        psesscvec = psesscvec_tmp / sum(psesscvec_tmp)

        #return psesscvec
        return [round(f,4) for f in psesscvec]

