#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: Nackel
"""
import index_list
import numpy as np
from itertools import combinations_with_replacement, permutations, product
import util

def get_triplet_matrix(fastas):
    '''This is a complete process in triplet,aim to generate feature vectors.
    :param filename: Name of inputfile.
    :return: Feature matrix through Triplet.
    '''
    letter = ["(","."]
    alphabet = 'AGCU'     #Don't change the alphabetical, or the order of features will change.
    tripletdict = get_triplet_dict(letter, 3, alphabet)
    features = []
    for seqss in fastas:
        vector = get_triplet_vector(seqss, tripletdict)
        features.append(vector)
    return features

def get_triplet_vector(seqss,patterndict):
    '''This is a process in triplet,aim to generate feature vector.

    :param seqss: a seqss object.
    :param patterndict: All the features, dictionary.
    :return: Feature vector through Triplet.
    '''
    vector=np.zeros((1,len(patterndict)))
    sequence, sstructure = util.delete_free_base(seqss)
    sequence, sstructure = util.delete_loop(seqss)

    for i in range(len(seqss.sequence)):
        letter =seqss.sequence[i]
        middle = seqss.sstructure[i]
        if i == 0:
            near_left = "."
            near_right = seqss.sstructure[i+1]
        elif i == len(seqss.sequence)-1:
            near_left = seqss.sstructure[i-1]
            near_right = "."
        else:
            near_left = seqss.sstructure[i-1]
            near_right = seqss.sstructure[i+1]
        #rectify the empty loop structure
        if middle == '(' and near_right == ')':
            near_right = '.'
        if middle == ')' and near_left == '(':
            near_left = '.'

        letter_sstruc_comb = letter+near_left+middle+near_right
        letter_sstruc_comb_r = letter_sstruc_comb.replace(')', '(')
        position = patterndict.get(letter_sstruc_comb_r)
        vector[0, position] += 1
        #print letter_sstruc_comb ,position
    #return list (vector[0])
    return [round(f,3) for f in list(vector[0]/sum(vector[0]))]


def get_triplet_dict(letter, k, alphabet=index_list.RNA):
    """Generate a dictionary of all possible triplet pattern.
    :param letter: A list that contains all the possible characters in the secondary structure. eg:['.','(']
    :param k: The length of k-mer.
    :param alphabet: A string that contains all the possible characters in the sequence.
    :return: A triplet dictionary.
    """
    kmerlst = util.get_kmer_lst(letter, k)
    kmerlst.reverse()
    tripletlst = [''.join(ele) for ele in product(list(alphabet), kmerlst)]
    #tripletlst = np.sort(tripletlst)
    tripletdict = {tripletlst[i]: i for i in range(len(tripletlst))}
    return tripletdict