#! /usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'Fule Liu'

import index_list,os,pickle,sys,re
from math import sqrt
from math import pow
import numpy as np
from itertools import combinations_with_replacement, permutations, product


def make_kmer_list(k, alphabet):
    """
    Generate kmer list.
    """
    if k < 0:
        print("Error, k must be an inter and larger than 0.")

    kmers = []
    for i in range(1, k + 1):
        if len(kmers) == 0:
            kmers = list(alphabet)
        else:
            new_kmers = []
            for kmer in kmers:
                for c in alphabet:
                    new_kmers.append(kmer + c)
            kmers = new_kmers

    return kmers

def frequency(tol_str, tar_str):
    """Generate the frequency of tar_str in tol_str.

    :param tol_str: mother string.
    :param tar_str: substring.
    """
    i, j, tar_count = 0, 0, 0
    len_tol_str = len(tol_str)
    len_tar_str = len(tar_str)
    while i < len_tol_str and j < len_tar_str:
        if tol_str[i] == tar_str[j]:
            i += 1
            j += 1
            if j >= len_tar_str:
                tar_count += 1
                i = i - j + 1
                j = 0
        else:
            i = i - j + 1
            j = 0

    return tar_count

def getKmerDict(alphabet, k):
    kmerlst = []
    partkmers = list(combinations_with_replacement(alphabet, k))
    for element in partkmers:
        elelst = set(permutations(element, k))
        strlst = [''.join(ele) for ele in elelst]
        kmerlst += strlst
    kmerlst = np.sort(kmerlst)
    kmerdict = {kmerlst[i]:i for i in range(len(kmerlst))}
    return kmerdict

def normalize_index(phyche_index, alphabet, is_convert_dict=False):
    """Normalize the physicochemical index."""
    normalize_phyche_value = []
    for phyche_value in phyche_index:
        average_phyche_value = sum(phyche_value) * 1.0 / len(phyche_value)
        sd_phyche = standard_deviation(phyche_value)
        normalize_phyche_value.append([round((e - average_phyche_value) / sd_phyche, 2) for e in phyche_value])

    if is_convert_dict is True:
        return convert_phyche_index_to_dict(normalize_phyche_value, alphabet)

    print(normalize_phyche_value)
    return normalize_phyche_value

def standard_deviation(value_list):
    """Return standard deviation."""
    n = len(value_list)
    average_value = sum(value_list) * 1.0 / n
    return sqrt(sum([pow(e - average_value, 2) for e in value_list]) * 1.0 / (n - 1))

def convert_phyche_index_to_dict(phyche_index, alphabet):
    """Convert phyche index from list to dict."""
    # for e in phyche_index:
    #     print e
    len_index_value = len(phyche_index[0])
    k = 0
    for i in range(1, 10):
        if len_index_value < 4**i:
            sys.exit("Sorry, the number of each index value is must be 4^k.")
        if len_index_value == 4**i:
            k = i
            break
    kmer_list = make_kmer_list(k, alphabet)
    # print kmer_list
    len_kmer = len(kmer_list)
    phyche_index_dict = {}
    for kmer in kmer_list:
        phyche_index_dict[kmer] = []
    # print phyche_index_dict
    phyche_index = list(zip(*phyche_index))
    for i in range(len_kmer):
        phyche_index_dict[kmer_list[i]] = list(phyche_index[i])

    return phyche_index_dict

def read_index(index_file):
    with open(index_file) as f_ind:
        lines = f_ind.readlines()
        ind_list = [index.rstrip() for index in lines]
        return ind_list

def getSpecificValue(olinuc, olinucs, prop, supInfo):
    # olinucs = getNucSeq(SupFileName).split(",")
    olinucs = olinucs.split(",")
    values = getValues(prop, supInfo).rstrip()
    values = values.split(",")
    # valueS = [float(x) for x in values.split(",")]
    count = olinucs.index(olinuc)
    value = values[count]
    return float(value)

def avgP(seq, olinucs, length, k, prop, supInfo):
    limit = length - k + 1
    i = 1
    sum = 0
    while i < limit or i == limit:
        # value = hn(seq[i - 1],prop,SupFileName)
        value = getSpecificValue(seq[i - 1], olinucs, prop, supInfo)
        sum = sum + value
        i = i + 1
    sum = sum / limit
    return sum

def getValues(prop, supInfo):
    values = ""
    name = re.search(prop, supInfo)
    if name:
        strr = prop + '\s*\,(.+)'
        b = re.search(strr, supInfo)
        if b:
            values = b.group(1)
    return values


def sepSequence(seq, k):
    i = k - 1
    seqq = []
    while i < len(seq):
        j = 0
        nuc = ''
        while j < k:
            nuc = seq[i - j] + nuc
            j = j + 1
        seqq.append(nuc)
        i += 1
    return seqq

def get_parallel_factor(k, lamada, sequence, phyche_value, alphabet):
    """Get the corresponding factor theta list."""
    theta = []
    l = len(sequence)

    for i in range(1, lamada + 1):
        temp_sum = 0.0
        for j in range(0, l - k - i + 1):
            nucleotide1 = sequence[j: j + k]
            nucleotide2 = sequence[j + i: j + i + k]
            if alphabet == index_list.DNA or alphabet == index_list.RNA:
                temp_sum += parallel_cor_function(nucleotide1, nucleotide2, phyche_value)
            elif alphabet == index_list.PROTEIN:
                temp_sum += pro_cor_fun1(nucleotide1, nucleotide2, phyche_value)

        theta.append(temp_sum / (l - k - i + 1))

    return theta

def parallel_cor_function(nucleotide1, nucleotide2, phyche_index):
    """Get the cFactor.(Type1)"""
    temp_sum = 0.0
    phyche_index_values = list(phyche_index.values())
    len_phyche_index = len(phyche_index_values[0])
    for u in range(len_phyche_index):
        temp_sum += pow(float(phyche_index[nucleotide1][u]) - float(phyche_index[nucleotide2][u]), 2)

    return temp_sum / len_phyche_index

def pro_cor_fun1(ri, rj, aaindex_list):
    _sum = 0.0
    len_index = len(aaindex_list)
    if isinstance(aaindex_list[0], tuple):
        for (head, index_dict) in aaindex_list:
            _sum += pow(index_dict[ri] - index_dict[rj], 2)
    else:
        for aaindex in aaindex_list:
            _sum += pow(aaindex.index_dict[ri] - aaindex.index_dict[rj], 2)
    return _sum / len_index

def get_series_factor(k, lamada, sequence, phyche_value, alphabet):
    """Get the corresponding series factor theta list."""
    theta = []
    l_seq = len(sequence)
    if alphabet == index_list.DNA or alphabet == index_list.RNA:
        temp_values = list(phyche_value.values())
        max_big_lamada = len(temp_values[0])
    elif alphabet == index_list.PROTEIN:
        max_big_lamada = len(phyche_value)

    for small_lamada in range(1, lamada + 1):
        for big_lamada in range(max_big_lamada):
            temp_sum = 0.0
            for i in range(0, l_seq - k - small_lamada + 1):
                nucleotide1 = sequence[i: i + k]
                nucleotide2 = sequence[i + small_lamada: i + small_lamada + k]
                if alphabet == index_list.DNA or alphabet == index_list.RNA:
                    temp_sum += series_cor_function(nucleotide1, nucleotide2, big_lamada, phyche_value)
                elif alphabet == index_list.PROTEIN:
                    temp_sum += pro_cor_fun2(nucleotide1, nucleotide2, phyche_value[big_lamada])

            theta.append(temp_sum / (l_seq - k - small_lamada + 1))

    return theta

def series_cor_function(nucleotide1, nucleotide2, big_lamada, phyche_value):
    """Get the series correlation Factor(Type 2)."""
    return float(phyche_value[nucleotide1][big_lamada]) * float(phyche_value[nucleotide2][big_lamada])

def pro_cor_fun2(ri, rj, aaindex):
    return aaindex.index_dict[ri] * aaindex.index_dict[rj]

def get_kmer_lst(letterlst, k):
    """Generate a list of all possible k-mer pattern.

    :param letter: A list that contains all the possible letters in the sequence.
    :param k: The length of k-mer.
    :return: A kmer list.
    """
    kmerlst = []
    letter_set = set(letterlst)
    letter = [''.join(i) for i in letter_set]

    partkmers = list(combinations_with_replacement(letter, k))

    for element in partkmers:
        elelst = set(permutations(element, k))
        strlst = [''.join(ele) for ele in elelst]
        kmerlst += strlst
    kmerlst = np.sort(kmerlst)
    return list(kmerlst)

def delete_free_base(seqss):
    """Delete free base based on secondary structure to produce a new sequence and secondary structure. New sequence and secondary structure is a substring of the original sequence and secondary structure.
    :param seqss: a seqss object.
    :return: A new sequence and sstructure,string.
    """
    left_pos = seqss.sstructure.index('(')
    right_pos = seqss.sstructure.rindex(')')
    return seqss.sequence[left_pos:right_pos+1], seqss.sstructure[left_pos:right_pos+1]


def delete_loop(seqss):
    """Delete loop(hairpin) based on secondary structure to produce a new sequence and secondary structure. New sequence and secondary structure is a substring of the original sequence and secondary structure.

    :param seqss: a seqss object.
    :return: A new sequence and sstructure,string.
    """
    loop_re = r'(\(\.+\))'
    loop_list = re.findall(loop_re, seqss.sstructure)
    for loop in loop_list:
        pos = seqss.sstructure.index(loop)
        length = len(loop)
        sstructure_dl = seqss.sstructure[:pos+1] + seqss.sstructure[pos+length-1:]
        sequence_dl = seqss.sequence[:pos+1] + seqss.sequence[pos+length-1:]
    return sequence_dl, sstructure_dl

def get_corresp_sequence(seqss):
    sstructure_lst = list(seqss.sstructure)
    if len(seqss.sequence)==len(seqss.sstructure):
        s = []
        for i in range(len(seqss.sequence)):
            if seqss.sstructure[i] == "(":
                s.append(i)
            if seqss.sstructure[i] ==")":
                pos = s.pop()
                sstructure_lst[pos] = seqss.sequence[i]
                sstructure_lst[i] = seqss.sequence[pos]
        sstructure_new = ''.join(sstructure_lst)
        return sstructure_new
    else:
        error_info = seqss.name+'The length of sequence is not equal to the length of its secondary structure.'
        sys.stderr.write(error_info)

def get_pseknc_vector(seqss, k, letter_list = ['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G']):
    '''This is a process in PseKNC, aim to generate feature vector.

    :param seqss: a seqss object.
    :param psekncdict: All the features, dictionary.
    :param k: The length of K-tuple nucleotide composition.
    :param letter_list: default   ['A', 'C', 'G', 'U', 'A-U', 'U-A', 'G-C', 'C-G', 'G-U', 'U-G'].
    :return: Feature vector through PseKNC.
    '''
    psekncdict = get_pseknc_dict(letter_list, k)
    vector = np.zeros((1, len(psekncdict)))
    correspseq = get_corresp_sequence(seqss)
    pattern = zip(list(seqss.sequence),list(correspseq))

    for i in xrange(len(pattern)-k+1):
        stem = []
        for x, y in pattern[i:i+k]:
            if x == '.' or y == '.':
                if x == '.':
                    stem.append(y)
                else:
                    stem.append(x)
            else:
                stem.append(x + '-' + y)
        stem_tuple= tuple(stem)
        position = psekncdict.get(stem_tuple)
        vector[0, position] += 1
        #print stem_tuple,position
    #return vector[0]
    return list(vector[0]/sum(vector[0]))
    #return [round(f,4) for f in list(vector[0]/sum(vector[0]))]

def calculate_theta(seqss, j):
    '''calculate theta
    :param seqss: a seqss object.
    :param j: the counted rank (or tier) of the structural correlation along a RNA chain.
    :return: theta.
    '''
    if j >= len(seqss.sequence):
        error_info = 'Error occured in '+seqss.id +', r should be less than the length of the sequence.'
        sys.stderr.write(error_info)
    else:
        correspseq = get_corresp_sequence(seqss)
        pattern = zip(list(seqss.sequence), list(correspseq))
        stem=[]
        for x,y in pattern:
            if x == '.' or y == '.':
                if x == '.':
                    stem.append(y)
                else:
                    stem.append(x)
            else:
                stem.append(x + '-' + y)
        freevalue_vector = []
        for i in stem:
            if i == 'A-U' or i == 'U-A':
               freevalue_vector.append(-2)
            elif i == 'C-G' or i == 'G-C':
                freevalue_vector.append(-3)
            elif i == 'U-G' or i == 'G-U':
                freevalue_vector.append(-1)
            else:
                freevalue_vector.append(0)

        s=0.0
        for i in range(len(freevalue_vector)-j):
            s += (freevalue_vector[i] - freevalue_vector[i+j]) ** 2
            #print i,i+j
            #print (freevalue_vector[i] - freevalue_vector[i+r])
        #print s,len(freevalue_vector)-r
        return s / (len(freevalue_vector)-j)

def get_pseknc_dict(letter_list, k):
    """Generate a dictionary of all possible PseKNC pattern.
    :param letter: A list that contains all the possible characters in an RNA sequence. eg:['A','C','G','U']
    :param k: The length of K-tuple nucleotide composition.
    :return: A PseKNC pattern dictionary.
    """
    pseknclst = []
    part_psessc = list(combinations_with_replacement(letter_list, k))
    for element in part_psessc:
        elelst = set(permutations(element, k))
        pseknclst += elelst
    pseknclst.sort()
    psekncdict = {pseknclst[i]:i for i in range(len(pseknclst))}
    return psekncdict